/*
 * This file is part of rasdaman community.
 *
 * Rasdaman community is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Rasdaman community is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with rasdaman community.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright 2003, 2004, 2005, 2006, 2007, 2008, 2009 Peter Baumann / rasdaman GmbH.
 *
 * For more information please see <http://www.rasdaman.org>
 * or contact Peter Baumann via <baumann@rasdaman.com>.
 /

 /**
 * A literal query trait provides functionality for defining string queries containing parameters that can be changed.
 * e.g "SELECT @col FROM @col WHERE @cond"
 * @module {Rj.query}
 * @author Alex Dumitru <m.dumitru@jacobs-university.de>
 * @author Vlad Merticariu <v.merticariu@jacobs-university.de>
 * @version 3.0.0
 */
FlancheJs.defineTrait("Rj.query.LiteralQuery", {

  properties:{
    /**
     * @property {String} query - the literal query e.g. "SELECT @col FROM @col"
     */
    query    :{
      value:""
    },
    /**
     * @property {Object} variables - an object of form {variable : value} e.g. {"@col" : "mr"}
     */
    variables:{
      value:{}
    }
  },

  methods:{
    /**
     * Sets a variable to a certain value
     * @param {String} varName
     * @param {String} value
     */
    setVariable:function (varName, value) {
      this.$variables[varName] = value;
    },

    /**
     * Returns a variable's value.
     * @param {String} varName
     * @return {String}
     */
    getVariable:function (varName) {
      return this.getVariables()[varName];
    },

    /**
     * Returns the string representation after the query is expanded by replacing the vars
     * @return {String}
     */
    toString:function () {
      return this._expand();
    },
    /**
     * Indicates if all the variables in the query are set
     * @return {Boolean}
     */
    isReady: function(){
      var variables = this.getVariables();
      for(var variable in variables){
        if(variables[variable] == undefined){
          return false;
        }
      }
      return true;
    }
  },

  internals:{
    expand:function () {
      var query = this.getQuery();
      var variables = this.getVariables();
      for (var variable in variables) {
        if(variables[variable] == undefined){
          Rj.util.ErrorManager.reportError('In Rj.query.LiteralQuery, expand(): Variables must be defined!', true);
        }
        var current = variables[variable].toString();
        query = query.split(variable).join(current);
      }
      return query;
    }
  }
});